@@ -10,19 +10,28 @@ Created on Wed Oct 30 14:01:27 2013
 
 Author: Josef Perktold
 """
-from statsmodels.compat.pandas import assert_series_equal, assert_index_equal
-from statsmodels.compat.python import range
-from statsmodels.compat.platform import PLATFORM_OSX
+from statsmodels.compat.pandas import assert_index_equal, assert_series_equal
+from statsmodels.compat.platform import (
+    PLATFORM_LINUX32,
+    PLATFORM_OSX,
+    PLATFORM_WIN32,
+)
+from statsmodels.compat.scipy import SCIPY_GT_14
 
 import numpy as np
+from numpy.testing import (
+    assert_,
+    assert_allclose,
+    assert_array_equal,
+    assert_equal,
+)
 import pandas as pd
 import pytest
-import statsmodels.api as sm
-import statsmodels.tools._testing as smt
-from statsmodels.formula.api import ols, glm
 
-from numpy.testing import (assert_, assert_allclose, assert_equal,
-                           assert_array_equal)
+import statsmodels.api as sm
+from statsmodels.formula.api import glm, ols
+import statsmodels.tools._testing as smt
+from statsmodels.tools.sm_exceptions import HessianInversionWarning
 
 
 class CheckGenericMixin(object):
@@ -107,17 +116,22 @@ class CheckGenericMixin(object):
         assert_equal(res1.bse[drop_index], 0)
         # OSX has many slight failures on this test
         tol = 1e-8 if PLATFORM_OSX else 1e-10
-        assert_allclose(res1.tvalues[keep_index_p], res2.tvalues, rtol=tol,
-                        atol=tol)
-        assert_allclose(res1.pvalues[keep_index_p], res2.pvalues, rtol=tol,
-                        atol=tol)
+        tvals1 = res1.tvalues[keep_index_p]
+        assert_allclose(tvals1, res2.tvalues, rtol=tol, atol=tol)
+
+        # See gh5993
+        if PLATFORM_LINUX32 or SCIPY_GT_14:
+            pvals1 = res1.pvalues[keep_index_p]
+        else:
+            pvals1 = res1.pvalues[keep_index_p]
+        assert_allclose(pvals1, res2.pvalues, rtol=tol, atol=tol)
 
         if hasattr(res1, 'resid'):
-            # discrete models, Logit don't have `resid` yet
+            # discrete models, Logit do not have `resid` yet
             # atol discussion at gh-5158
             rtol = 1e-10
             atol = 1e-12
-            if PLATFORM_OSX:
+            if PLATFORM_OSX or PLATFORM_WIN32:
                 # GH 5628
                 rtol = 1e-8
                 atol = 1e-10
@@ -179,6 +193,11 @@ class CheckGenericMixin(object):
             keep_index_p += list(range(k_vars, k_vars + mod.k_extra))
             k_extra = mod.k_extra
 
+        # TODO: Can we choose a test case without this issue?
+        #  If not, should we be getting this warning for all
+        #  model subclasses?
+        warn_cls = HessianInversionWarning if isinstance(mod, sm.GLM) else None
+
         cov_types = ['nonrobust', 'HC0']
 
         for cov_type in cov_types:
@@ -190,28 +209,31 @@ class CheckGenericMixin(object):
 
             if use_start_params:
                 start_params = np.zeros(k_vars + k_extra)
-                method =  self.results.mle_settings['optimizer']
+                method = self.results.mle_settings['optimizer']
                 # string in `method` is not mutable, so no need for copy
-                sp =  self.results.mle_settings['start_params'].copy()
+                sp = self.results.mle_settings['start_params'].copy()
                 if self.transform_index is not None:
                     # work around internal transform_params, currently in NB
                     sp[self.transform_index] = np.exp(sp[self.transform_index])
 
                 start_params[keep_index_p] = sp
-                res1 = mod._fit_collinear(cov_type=cov_type,
-                                          start_params=start_params,
-                                          method=method, disp=0)
+                with pytest.warns(warn_cls):
+                    res1 = mod._fit_collinear(cov_type=cov_type,
+                                              start_params=start_params,
+                                              method=method, disp=0)
                 if cov_type != 'nonrobust':
                     # reestimate original model to get robust cov
-                    res2 = self.results.model.fit(cov_type=cov_type,
-                                                  start_params=sp,
-                                                  method=method, disp=0)
+                    with pytest.warns(warn_cls):
+                        res2 = self.results.model.fit(cov_type=cov_type,
+                                                      start_params=sp,
+                                                      method=method, disp=0)
             else:
-                # more special casing RLM
-                if (isinstance(self.results.model, (sm.RLM))):
-                    res1 = mod._fit_collinear()
-                else:
-                    res1 = mod._fit_collinear(cov_type=cov_type)
+                with pytest.warns(warn_cls):
+                    # more special casing RLM
+                    if (isinstance(self.results.model, (sm.RLM))):
+                        res1 = mod._fit_collinear()
+                    else:
+                        res1 = mod._fit_collinear(cov_type=cov_type)
                 if cov_type != 'nonrobust':
                     # reestimate original model to get robust cov
                     res2 = self.results.model.fit(cov_type=cov_type)
@@ -239,12 +261,18 @@ class CheckGenericMixin(object):
             assert_allclose(res1.params[drop_index], 0, rtol=1e-10)
             assert_allclose(res1.bse[keep_index_p], res2.bse, rtol=1e-8)
             assert_allclose(res1.bse[drop_index], 0, rtol=1e-10)
-            assert_allclose(res1.tvalues[keep_index_p], res2.tvalues, rtol=5e-8)
-            assert_allclose(res1.pvalues[keep_index_p], res2.pvalues,
-                            rtol=1e-6, atol=1e-30)
+            tvals1 = res1.tvalues[keep_index_p]
+            assert_allclose(tvals1, res2.tvalues, rtol=5e-8)
+
+            # See gh5993
+            if PLATFORM_LINUX32 or SCIPY_GT_14:
+                pvals1 = res1.pvalues[keep_index_p]
+            else:
+                pvals1 = res1.pvalues[keep_index_p]
+            assert_allclose(pvals1, res2.pvalues, rtol=1e-6, atol=1e-30)
 
             if hasattr(res1, 'resid'):
-                # discrete models, Logit don't have `resid` yet
+                # discrete models, Logit do not have `resid` yet
                 assert_allclose(res1.resid, res2.resid, rtol=1e-5, atol=1e-10)
 
             ex = res1.model.exog.mean(0)
@@ -337,7 +365,9 @@ class TestGenericNegativeBinomial(CheckGenericMixin):
     def setup(self):
         #fit for each test, because results will be changed by test
         np.random.seed(987689)
-        data = sm.datasets.randhie.load(as_pandas=False)
+        data = sm.datasets.randhie.load()
+        data.exog = np.asarray(data.exog)
+        data.endog = np.asarray(data.endog)
         exog = sm.add_constant(data.exog, prepend=False)
         mod = sm.NegativeBinomial(data.endog, exog)
         start_params = np.array([-0.05783623, -0.26655806,  0.04109148, -0.03815837,
@@ -358,7 +388,10 @@ class TestGenericLogit(CheckGenericMixin):
         model = sm.Logit(y_bin, x)  #, exposure=np.ones(nobs), offset=np.zeros(nobs)) #bug with default
         # use start_params to converge faster
         start_params = np.array([-0.73403806, -1.00901514, -0.97754543, -0.95648212])
-        self.results = model.fit(start_params=start_params, method='bfgs', disp=0)
+        with pytest.warns(FutureWarning,
+                          match="Keyword arguments have been passed"):
+            self.results = model.fit(start_params=start_params,
+                                     method='bfgs', disp=0, tol=1e-5)
 
 
 class TestGenericRLM(CheckGenericMixin):
