@@ -16,24 +16,36 @@ Judge, ... (1985): The Theory and Practise of Econometrics
 Author: josefpktd
 License: BSD
 """
-from statsmodels.compat.pandas import deprecate_kwarg
+from statsmodels.compat.pandas import Appender
 
 import numpy as np
-from scipy import signal, optimize, linalg
+from scipy import linalg, optimize, signal
 
-from statsmodels.compat.pandas import Appender
-from statsmodels.tools.docstring import remove_parameters, Docstring
+from statsmodels.tools.docstring import Docstring, remove_parameters
 from statsmodels.tools.validation import array_like
 
-__all__ = ['arma_acf', 'arma_acovf', 'arma_generate_sample',
-           'arma_impulse_response', 'arma2ar', 'arma2ma', 'deconvolve',
-           'lpol2index', 'index2lpol']
+__all__ = [
+    "arma_acf",
+    "arma_acovf",
+    "arma_generate_sample",
+    "arma_impulse_response",
+    "arma2ar",
+    "arma2ma",
+    "deconvolve",
+    "lpol2index",
+    "index2lpol",
+]
 
 
-# Remove after 0.11
-@deprecate_kwarg('sigma', 'scale')
-def arma_generate_sample(ar, ma, nsample, scale=1, distrvs=None,
-                         axis=0, burnin=0):
+NONSTATIONARY_ERROR = """\
+The model's autoregressive parameters (ar) indicate that the process
+ is non-stationary. arma_acovf can only be used with stationary processes.
+"""
+
+
+def arma_generate_sample(
+    ar, ma, nsample, scale=1, distrvs=None, axis=0, burnin=0
+):
     """
     Simulate data from an ARMA.
 
@@ -52,8 +64,8 @@ def arma_generate_sample(ar, ma, nsample, scale=1, distrvs=None,
     scale : float
         The standard deviation of noise.
     distrvs : function, random number generator
-        A function that generates the random numbers, and takes sample size
-        as argument. The default is np.random.randn.
+        A function that generates the random numbers, and takes ``size``
+        as argument. The default is np.random.standard_normal.
     axis : int
         See nsample for details.
     burnin : int
@@ -82,11 +94,11 @@ def arma_generate_sample(ar, ma, nsample, scale=1, distrvs=None,
     >>> ar = np.r_[1, -arparams] # add zero-lag and negate
     >>> ma = np.r_[1, maparams] # add zero-lag
     >>> y = sm.tsa.arma_generate_sample(ar, ma, 250)
-    >>> model = sm.tsa.ARMA(y, (2, 2)).fit(trend='nc', disp=0)
+    >>> model = sm.tsa.ARIMA(y, (2, 0, 2), trend='n').fit(disp=0)
     >>> model.params
     array([ 0.79044189, -0.23140636,  0.70072904,  0.40608028])
     """
-    distrvs = np.random.normal if distrvs is None else distrvs
+    distrvs = np.random.standard_normal if distrvs is None else distrvs
     if np.ndim(nsample) == 0:
         nsample = [nsample]
     if burnin:
@@ -107,7 +119,7 @@ def arma_generate_sample(ar, ma, nsample, scale=1, distrvs=None,
 
 def arma_acovf(ar, ma, nobs=10, sigma2=1, dtype=None):
     """
-    Theoretical autocovariance function of ARMA process.
+    Theoretical autocovariances of stationary ARMA processes
 
     Parameters
     ----------
@@ -143,13 +155,15 @@ def arma_acovf(ar, ma, nobs=10, sigma2=1, dtype=None):
     m = max(p, q) + 1
 
     if sigma2.real < 0:
-        raise ValueError('Must have positive innovation variance.')
+        raise ValueError("Must have positive innovation variance.")
 
     # Short-circuit for trivial corner-case
     if p == q == 0:
         out = np.zeros(nobs, dtype=dtype)
         out[0] = sigma2
         return out
+    elif p > 0 and np.max(np.abs(np.roots(ar))) >= 1:
+        raise ValueError(NONSTATIONARY_ERROR)
 
     # Get the moving average representation coefficients that we need
     ma_coeffs = arma2ma(ar, ma, lags=m)
@@ -160,25 +174,27 @@ def arma_acovf(ar, ma, nobs=10, sigma2=1, dtype=None):
     b = np.zeros((m, 1), dtype=dtype)
     # We need a zero-right-padded version of ar params
     tmp_ar = np.zeros(m, dtype=dtype)
-    tmp_ar[:p + 1] = ar
+    tmp_ar[: p + 1] = ar
     for k in range(m):
-        A[k, :(k + 1)] = tmp_ar[:(k + 1)][::-1]
-        A[k, 1:m - k] += tmp_ar[(k + 1):m]
-        b[k] = sigma2 * np.dot(ma[k:q + 1], ma_coeffs[:max((q + 1 - k), 0)])
+        A[k, : (k + 1)] = tmp_ar[: (k + 1)][::-1]
+        A[k, 1 : m - k] += tmp_ar[(k + 1) : m]
+        b[k] = sigma2 * np.dot(ma[k : q + 1], ma_coeffs[: max((q + 1 - k), 0)])
     acovf = np.zeros(max(nobs, m), dtype=dtype)
-    acovf[:m] = np.linalg.solve(A, b)[:, 0]
+    try:
+        acovf[:m] = np.linalg.solve(A, b)[:, 0]
+    except np.linalg.LinAlgError:
+        raise ValueError(NONSTATIONARY_ERROR)
 
     # Iteratively apply (BD, eq. 3.3.9) to solve for remaining autocovariances
     if nobs > m:
         zi = signal.lfiltic([1], ar, acovf[:m:][::-1])
-        acovf[m:] = signal.lfilter([1], ar, np.zeros(nobs - m, dtype=dtype),
-                                   zi=zi)[0]
+        acovf[m:] = signal.lfilter(
+            [1], ar, np.zeros(nobs - m, dtype=dtype), zi=zi
+        )[0]
 
     return acovf[:nobs]
 
 
-# Remove after 0.11
-@deprecate_kwarg('nobs', 'lags')
 def arma_acf(ar, ma, lags=10):
     """
     Theoretical autocorrelation function of an ARMA process.
@@ -207,8 +223,6 @@ def arma_acf(ar, ma, lags=10):
     return acovf / acovf[0]
 
 
-# Remove after 0.11
-@deprecate_kwarg('nobs', 'lags')
 def arma_pacf(ar, ma, lags=10):
     """
     Theoretical partial autocorrelation function of an ARMA process.
@@ -237,7 +251,7 @@ def arma_pacf(ar, ma, lags=10):
     apacf = np.zeros(lags)
     acov = arma_acf(ar, ma, lags=lags + 1)
 
-    apacf[0] = 1.
+    apacf[0] = 1.0
     for k in range(2, lags + 1):
         r = acov[:k]
         apacf[k - 1] = linalg.solve(linalg.toeplitz(r[:-1]), r[1:])[-1]
@@ -283,13 +297,14 @@ def arma_periodogram(ar, ma, worN=None, whole=0):
     if np.any(np.isnan(h)):
         # this happens with unit root or seasonal unit root'
         import warnings
-        warnings.warn('Warning: nan in frequency response h, maybe a unit '
-                      'root', RuntimeWarning)
+
+        warnings.warn(
+            "Warning: nan in frequency response h, maybe a unit " "root",
+            RuntimeWarning,
+        )
     return w, sd
 
 
-# Remove after 0.11
-@deprecate_kwarg('nobs', 'leads')
 def arma_impulse_response(ar, ma, leads=100):
     """
     Compute the impulse response function (MA representation) for ARMA process.
@@ -345,12 +360,10 @@ def arma_impulse_response(ar, ma, leads=100):
             0.63488   ,  0.507904  ,  0.4063232 ,  0.32505856,  0.26004685])
     """
     impulse = np.zeros(leads)
-    impulse[0] = 1.
+    impulse[0] = 1.0
     return signal.lfilter(ma, ar, impulse)
 
 
-# Remove after 0.11
-@deprecate_kwarg('nobs', 'lags')
 def arma2ma(ar, ma, lags=100):
     """
     A finite-lag approximate MA representation of an ARMA process.
@@ -376,8 +389,6 @@ def arma2ma(ar, ma, lags=100):
     return arma_impulse_response(ar, ma, leads=lags)
 
 
-# Remove after 0.11
-@deprecate_kwarg('nobs', 'lags')
 def arma2ar(ar, ma, lags=100):
     """
     A finite-lag AR approximation of an ARMA process.
@@ -404,7 +415,7 @@ def arma2ar(ar, ma, lags=100):
 
 
 # moved from sandbox.tsa.try_fi
-def ar2arma(ar_des, p, q, n=20, mse='ar', start=None):
+def ar2arma(ar_des, p, q, n=20, mse="ar", start=None):
     """
     Find arma approximation to ar process.
 
@@ -451,9 +462,9 @@ def ar2arma(ar_des, p, q, n=20, mse='ar', start=None):
 
     # p,q = pq
     def msear_err(arma, ar_des):
-        ar, ma = np.r_[1, arma[:p - 1]], np.r_[1, arma[p - 1:]]
+        ar, ma = np.r_[1, arma[: p - 1]], np.r_[1, arma[p - 1 :]]
         ar_approx = arma_impulse_response(ma, ar, n)
-        return (ar_des - ar_approx)  # ((ar - ar_approx)**2).sum()
+        return ar_des - ar_approx  # ((ar - ar_approx)**2).sum()
 
     if start is None:
         arma0 = np.r_[-0.9 * np.ones(p - 1), np.zeros(q - 1)]
@@ -461,13 +472,12 @@ def ar2arma(ar_des, p, q, n=20, mse='ar', start=None):
         arma0 = start
     res = optimize.leastsq(msear_err, arma0, ar_des, maxfev=5000)
     arma_app = np.atleast_1d(res[0])
-    ar_app = np.r_[1, arma_app[:p - 1]],
-    ma_app = np.r_[1, arma_app[p - 1:]]
+    ar_app = (np.r_[1, arma_app[: p - 1]],)
+    ma_app = np.r_[1, arma_app[p - 1 :]]
     return ar_app, ma_app, res
 
 
-_arma_docs = {'ar': arma2ar.__doc__,
-              'ma': arma2ma.__doc__}
+_arma_docs = {"ar": arma2ar.__doc__, "ma": arma2ma.__doc__}
 
 
 def lpol2index(ar):
@@ -486,7 +496,7 @@ def lpol2index(ar):
     index : ndarray
         index (lags) of lag polynomial with non-zero elements
     """
-    ar = array_like(ar, 'ar')
+    ar = array_like(ar, "ar")
     index = np.nonzero(ar)[0]
     coeffs = ar[index]
     return coeffs, index
@@ -533,6 +543,7 @@ def lpol_fima(d, n=20):
     """
     # hide import inside function until we use this heavily
     from scipy.special import gammaln
+
     j = np.arange(n)
     return np.exp(gammaln(d + j) - gammaln(j + 1) - gammaln(d))
 
@@ -561,8 +572,9 @@ def lpol_fiar(d, n=20):
     """
     # hide import inside function until we use this heavily
     from scipy.special import gammaln
+
     j = np.arange(n)
-    ar = - np.exp(gammaln(-d + j) - gammaln(j + 1) - gammaln(-d))
+    ar = -np.exp(gammaln(-d + j) - gammaln(j + 1) - gammaln(-d))
     ar[0] = 1
     return ar
 
@@ -628,7 +640,7 @@ def deconvolve(num, den, n=None):
         input = np.zeros(n, float)
         input[0] = 1
         quot = signal.lfilter(num, den, input)
-        num_approx = signal.convolve(den, quot, mode='full')
+        num_approx = signal.convolve(den, quot, mode="full")
         if len(num) < len(num_approx):  # 1d only ?
             num = np.concatenate((num, np.zeros(len(num_approx) - len(num))))
         rem = num - num_approx
@@ -636,9 +648,9 @@ def deconvolve(num, den, n=None):
 
 
 _generate_sample_doc = Docstring(arma_generate_sample.__doc__)
-_generate_sample_doc.remove_parameters(['ar', 'ma'])
-_generate_sample_doc.replace_block('Notes', [])
-_generate_sample_doc.replace_block('Examples', [])
+_generate_sample_doc.remove_parameters(["ar", "ma"])
+_generate_sample_doc.replace_block("Notes", [])
+_generate_sample_doc.replace_block("Examples", [])
 
 
 class ArmaProcess(object):
@@ -699,7 +711,7 @@ class ArmaProcess(object):
     >>> arma_process.arroots
     array([1.5-1.32287566j, 1.5+1.32287566j])
     >>> y = arma_process.generate_sample(250)
-    >>> model = sm.tsa.ARMA(y, (2, 2)).fit(trend='nc', disp=0)
+    >>> model = sm.tsa.ARIMA(y, (2, 0, 2), trend='n').fit(disp=0)
     >>> model.params
     array([ 0.79044189, -0.23140636,  0.70072904,  0.40608028])
 
@@ -713,17 +725,57 @@ class ArmaProcess(object):
     # TODO: Check unit root behavior
     def __init__(self, ar=None, ma=None, nobs=100):
         if ar is None:
-            ar = np.array([1.])
+            ar = np.array([1.0])
         if ma is None:
-            ma = np.array([1.])
-        self.ar = array_like(ar, 'ar')
-        self.ma = array_like(ma, 'ma')
+            ma = np.array([1.0])
+        self.ar = array_like(ar, "ar")
+        self.ma = array_like(ma, "ma")
         self.arcoefs = -self.ar[1:]
         self.macoefs = self.ma[1:]
         self.arpoly = np.polynomial.Polynomial(self.ar)
         self.mapoly = np.polynomial.Polynomial(self.ma)
         self.nobs = nobs
 
+    @classmethod
+    def from_roots(cls, maroots=None, arroots=None, nobs=100):
+        """
+        Create ArmaProcess from AR and MA polynomial roots.
+
+        Parameters
+        ----------
+        maroots : array_like
+            Roots for the MA polynomial
+            1 + theta_1*z + theta_2*z^2 + ..... + theta_n*z^n
+        arroots : array_like
+            Roots for the AR polynomial
+            1 - phi_1*z - phi_2*z^2 - ..... - phi_n*z^n
+        nobs : int, optional
+            Length of simulated time series. Used, for example, if a sample
+            is generated.
+
+        Returns
+        -------
+        ArmaProcess
+            Class instance initialized with arcoefs and macoefs.
+
+        Examples
+        --------
+        >>> arroots = [.75, -.25]
+        >>> maroots = [.65, .35]
+        >>> arma_process = sm.tsa.ArmaProcess.from_roots(arroots, maroots)
+        >>> arma_process.isstationary
+        True
+        >>> arma_process.isinvertible
+        True
+        """
+        arpoly = np.polynomial.polynomial.Polynomial.fromroots(arroots)
+        mapoly = np.polynomial.polynomial.Polynomial.fromroots(maroots)
+        # As from_coeffs will create a polynomial with constant 1/-1,(MA/AR)
+        # we need to scale the polynomial coefficients accordingly
+        return cls(np.r_[1, -np.asarray(-1 * arpoly.coef[1:] / arpoly.coef[0])],
+                   np.r_[1, np.asarray(mapoly.coef[1:] / mapoly.coef[0])],
+                   nobs=nobs)
+
     @classmethod
     def from_coeffs(cls, arcoefs=None, macoefs=None, nobs=100):
         """
@@ -759,18 +811,20 @@ class ArmaProcess(object):
         """
         arcoefs = [] if arcoefs is None else arcoefs
         macoefs = [] if macoefs is None else macoefs
-        return cls(np.r_[1, -np.asarray(arcoefs)],
-                   np.r_[1, np.asarray(macoefs)],
-                   nobs=nobs)
+        return cls(
+            np.r_[1, -np.asarray(arcoefs)],
+            np.r_[1, np.asarray(macoefs)],
+            nobs=nobs,
+        )
 
     @classmethod
     def from_estimation(cls, model_results, nobs=None):
         """
-        Create an ArmaProcess from the results of an ARMA estimation.
+        Create an ArmaProcess from the results of an ARIMA estimation.
 
         Parameters
         ----------
-        model_results : ARMAResults instance
+        model_results : ARIMAResults instance
             A fitted model.
         nobs : int, optional
             If None, nobs is taken from the results.
@@ -779,11 +833,18 @@ class ArmaProcess(object):
         -------
         ArmaProcess
             Class instance initialized from model_results.
+
+        See Also
+        --------
+        statsmodels.tsa.arima.model.ARIMA
+            The models class used to create the ArmaProcess
         """
-        arcoefs = model_results.arparams
-        macoefs = model_results.maparams
         nobs = nobs or model_results.nobs
-        return cls(np.r_[1, -arcoefs], np.r_[1, macoefs], nobs=nobs)
+        return cls(
+            model_results.polynomial_reduced_ar,
+            model_results.polynomial_reduced_ma,
+            nobs=nobs,
+        )
 
     def __mul__(self, oth):
         if isinstance(oth, self.__class__):
@@ -797,50 +858,55 @@ class ArmaProcess(object):
                 ar = (self.arpoly * arpolyoth).coef
                 ma = (self.mapoly * mapolyoth).coef
             except:
-                raise TypeError('Other type is not a valid type')
+                raise TypeError("Other type is not a valid type")
         return self.__class__(ar, ma, nobs=self.nobs)
 
     def __repr__(self):
-        msg = 'ArmaProcess({0}, {1}, nobs={2}) at {3}'
-        return msg.format(self.ar.tolist(), self.ma.tolist(),
-                          self.nobs, hex(id(self)))
+        msg = "ArmaProcess({0}, {1}, nobs={2}) at {3}"
+        return msg.format(
+            self.ar.tolist(), self.ma.tolist(), self.nobs, hex(id(self))
+        )
 
     def __str__(self):
-        return 'ArmaProcess\nAR: {0}\nMA: {1}'.format(self.ar.tolist(),
-                                                      self.ma.tolist())
+        return "ArmaProcess\nAR: {0}\nMA: {1}".format(
+            self.ar.tolist(), self.ma.tolist()
+        )
 
-    @Appender(remove_parameters(arma_acovf.__doc__, ['ar', 'ma', 'sigma2']))
+    @Appender(remove_parameters(arma_acovf.__doc__, ["ar", "ma", "sigma2"]))
     def acovf(self, nobs=None):
         nobs = nobs or self.nobs
         return arma_acovf(self.ar, self.ma, nobs=nobs)
 
-    @Appender(remove_parameters(arma_acf.__doc__, ['ar', 'ma']))
+    @Appender(remove_parameters(arma_acf.__doc__, ["ar", "ma"]))
     def acf(self, lags=None):
         lags = lags or self.nobs
         return arma_acf(self.ar, self.ma, lags=lags)
 
-    @Appender(remove_parameters(arma_pacf.__doc__, ['ar', 'ma']))
+    @Appender(remove_parameters(arma_pacf.__doc__, ["ar", "ma"]))
     def pacf(self, lags=None):
         lags = lags or self.nobs
         return arma_pacf(self.ar, self.ma, lags=lags)
 
-    @Appender(remove_parameters(arma_periodogram.__doc__, ['ar', 'ma', 'worN',
-                                                           'whole']))
+    @Appender(
+        remove_parameters(
+            arma_periodogram.__doc__, ["ar", "ma", "worN", "whole"]
+        )
+    )
     def periodogram(self, nobs=None):
         nobs = nobs or self.nobs
         return arma_periodogram(self.ar, self.ma, worN=nobs)
 
-    @Appender(remove_parameters(arma_impulse_response.__doc__, ['ar', 'ma']))
+    @Appender(remove_parameters(arma_impulse_response.__doc__, ["ar", "ma"]))
     def impulse_response(self, leads=None):
         leads = leads or self.nobs
         return arma_impulse_response(self.ar, self.ma, leads=leads)
 
-    @Appender(remove_parameters(arma2ma.__doc__, ['ar', 'ma']))
+    @Appender(remove_parameters(arma2ma.__doc__, ["ar", "ma"]))
     def arma2ma(self, lags=None):
         lags = lags or self.lags
         return arma2ma(self.ar, self.ma, lags=lags)
 
-    @Appender(remove_parameters(arma2ar.__doc__, ['ar', 'ma']))
+    @Appender(remove_parameters(arma2ar.__doc__, ["ar", "ma"]))
     def arma2ar(self, lags=None):
         lags = lags or self.lags
         return arma2ar(self.ar, self.ma, lags=lags)
@@ -911,7 +977,7 @@ class ArmaProcess(object):
         mainv = self.ma
         invertible = self.isinvertible
         if not invertible:
-            pr[np.abs(pr) < 1] = 1. / pr[np.abs(pr) < 1]
+            pr[np.abs(pr) < 1] = 1.0 / pr[np.abs(pr) < 1]
             pnew = np.polynomial.Polynomial.fromroots(pr)
             mainv = pnew.coef / pnew.coef[0]
 
@@ -921,7 +987,9 @@ class ArmaProcess(object):
             return mainv, invertible
 
     @Appender(str(_generate_sample_doc))
-    def generate_sample(self, nsample=100, scale=1., distrvs=None, axis=0,
-                        burnin=0):
-        return arma_generate_sample(self.ar, self.ma, nsample, scale, distrvs,
-                                    axis=axis, burnin=burnin)
+    def generate_sample(
+        self, nsample=100, scale=1.0, distrvs=None, axis=0, burnin=0
+    ):
+        return arma_generate_sample(
+            self.ar, self.ma, nsample, scale, distrvs, axis=axis, burnin=burnin
+        )
