@@ -1,7 +1,7 @@
 """
 Author: Samuel Scherrer
 """
-from statsmodels.compat.platform import PLATFORM_WIN, PLATFORM_LINUX32
+from statsmodels.compat.platform import PLATFORM_LINUX32, PLATFORM_WIN
 
 from itertools import product
 import json
@@ -456,7 +456,7 @@ def test_fit_vs_R(setup_model, reset_randomstate):
         start = params
     else:
         start = None
-    fit = model.fit(disp=True, tol=1e-8, start_params=start)
+    fit = model.fit(disp=True, pgtol=1e-8, start_params=start)
 
     # check log likelihood: we want to have a fit that is better, i.e. a fit
     # that has a **higher** log-likelihood
@@ -466,7 +466,7 @@ def test_fit_vs_R(setup_model, reset_randomstate):
     try:
         assert loglike >= loglike_R - 1e-4
     except AssertionError:
-        fit = model.fit(disp=True, tol=1e-8, start_params=params)
+        fit = model.fit(disp=True, pgtol=1e-8, start_params=params)
         loglike = fit.llf
         try:
             assert loglike >= loglike_R - 1e-4
@@ -691,12 +691,36 @@ def test_prediction_results(austourists_model_fit):
     assert len(summary["mean"].values) == 81
     assert np.all(~np.isnan(summary["mean"]))
 
-    # only out of sample prediction
+    # long out of sample, starting in-sample
     pred = austourists_model_fit.get_prediction(start=67, end=80)
     summary = pred.summary_frame()
     assert len(summary["mean"].values) == 14
     assert np.all(~np.isnan(summary["mean"]))
 
+    # long out of sample, starting at end of sample
+    pred = austourists_model_fit.get_prediction(start=68, end=80)
+    summary = pred.summary_frame()
+    assert len(summary["mean"].values) == 13
+    assert np.all(~np.isnan(summary["mean"]))
+
+    # long out of sample, starting just out of sample
+    pred = austourists_model_fit.get_prediction(start=69, end=80)
+    summary = pred.summary_frame()
+    assert len(summary["mean"].values) == 12
+    assert np.all(~np.isnan(summary["mean"]))
+
+    # long out of sample, starting long out of sample
+    pred = austourists_model_fit.get_prediction(start=79, end=80)
+    summary = pred.summary_frame()
+    assert len(summary["mean"].values) == 2
+    assert np.all(~np.isnan(summary["mean"]))
+
+    # long out of sample, `start`== `end`
+    pred = austourists_model_fit.get_prediction(start=80, end=80)
+    summary = pred.summary_frame()
+    assert len(summary["mean"].values) == 1
+    assert np.all(~np.isnan(summary["mean"]))
+
 
 @pytest.fixture
 def statespace_comparison(austourists):
@@ -743,11 +767,10 @@ def test_results_vs_statespace(statespace_comparison):
     )
 
     # compare diagnostics
-    with pytest.warns(FutureWarning):
-        assert_almost_equal(
-            ets_results.test_serial_correlation(method="ljungbox"),
-            statespace_results.test_serial_correlation(method="ljungbox"),
-        )
+    assert_almost_equal(
+        ets_results.test_serial_correlation(method="ljungbox"),
+        statespace_results.test_serial_correlation(method="ljungbox"),
+    )
     assert_almost_equal(
         ets_results.test_normality(method="jarquebera"),
         statespace_results.test_normality(method="jarquebera"),
@@ -759,8 +782,11 @@ def test_results_vs_statespace(statespace_comparison):
         method="breakvar"
     )[0]
     # het[0] is test statistic, het[1] p-value
-    assert_allclose(ets_het[0], statespace_het[0], rtol=0.2)
-    assert_allclose(ets_het[1], statespace_het[1], rtol=0.7)
+    if not PLATFORM_LINUX32:
+        # Skip on Linux-32 bit due to random failures. These values are not
+        # close at all in any way so it isn't clear what this is testing
+        assert_allclose(ets_het[0], statespace_het[0], rtol=0.2)
+        assert_allclose(ets_het[1], statespace_het[1], rtol=0.7)
 
 
 def test_prediction_results_vs_statespace(statespace_comparison):
@@ -981,3 +1007,43 @@ def test_exact_prediction_intervals(austourists_model_fit):
     fit.model = DummyModel("AAA")
     s_AAA = fit._relative_forecast_variance(steps)
     assert_almost_equal(s_AAdA, s_AAA, 2)
+
+
+def test_one_step_ahead(setup_model):
+    model, params, results_R = setup_model
+    model2 = ETSModel(
+        pd.Series(model.endog),
+        seasonal_periods=model.seasonal_periods,
+        error=model.error,
+        trend=model.trend,
+        seasonal=model.seasonal,
+        damped_trend=model.damped_trend,
+    )
+    res = model2.smooth(params)
+
+    fcast1 = res.forecast(steps=1)
+    fcast2 = res.forecast(steps=2)
+    assert_allclose(fcast1.iloc[0], fcast2.iloc[0])
+
+    pred1 = res.get_prediction(start=model2.nobs, end=model2.nobs,
+                               simulate_repetitions=2)
+    pred2 = res.get_prediction(start=model2.nobs, end=model2.nobs + 1,
+                               simulate_repetitions=2)
+    df1 = pred1.summary_frame(alpha=0.05)
+    df2 = pred1.summary_frame(alpha=0.05)
+    assert_allclose(df1.iloc[0, 0], df2.iloc[0, 0])
+
+
+@pytest.mark.parametrize("trend", [None, "add"])
+@pytest.mark.parametrize("seasonal", [None, "add"])
+@pytest.mark.parametrize("nobs", [9, 10])
+def test_estimated_initialization_short_data(oildata, trend, seasonal, nobs):
+    # GH 7319
+    res = ETSModel(
+        oildata[:nobs],
+        trend=trend,
+        seasonal=seasonal,
+        seasonal_periods=4,
+        initialization_method='estimated'
+    ).fit()
+    assert ~np.any(np.isnan(res.params))
